@@ -249,7 +249,7 @@ sub embed_header
     }
 
     # Print default options
-    print " <script src=\"$theme_webprefix/unauthenticated/js/defaults.js?" .
+    print " <script type='application/javascript' src=\"$theme_webprefix/unauthenticated/js/defaults.js?" .
       theme_version('timestamped') . "\"></script>\n";
 
     #
@@ -295,7 +295,7 @@ sub embed_header
         }
         return "$var=$quote_opening$rs$quote_closing;";
     };
-    print ' <script>';
+    print ' <script type="application/javascript">';
     print &$_sstj('theme_server_data_available_acls',    'get_acls_status',    'filemin');
     print &$_sstj('theme_server_data_available_selinux', 'get_selinux_status', 'filemin');
     print "</script>\n";
@@ -307,10 +307,15 @@ sub embed_header
     embed_tconfig();
 
     # Print object with language strings
-    print ' <script>';
+    print ' <script type="application/javascript">';
     print 'var v___theme_language = ' . get_theme_language();
     print "</script>\n";
 
+    # Print server original webprefix for proxy setups
+    print ' <script type="application/javascript">';
+    print 'const v___server_webprefix = "'.($gconfig{'webprefix'} || '').'"';
+    print "</script>\n";
+
     if ($args[2]) {
         load_devel_dependencies();
     }
@@ -336,7 +341,7 @@ sub embed_header
                     next;
                 }
 
-                print ' <script src="' .
+                print ' <script type="application/javascript" src="' .
                   $theme_webprefix .
                   '/unauthenticated/js/' .
                   $js .
@@ -350,7 +355,7 @@ sub embed_header
     } else {
         if ($args[2]) {
             foreach my $css (@theme_bundle_css) {
-                print ' <link href="' .
+                print ' <link type="text/css" href="' .
                   $theme_webprefix .
                   '/unauthenticated/css/' .
                   $css .
@@ -368,7 +373,7 @@ sub embed_header
         if ((length $theme_config{'settings_navigation_color'} && $theme_config{'settings_navigation_color'} ne 'blue') ||
             theme_night_mode())
         {
-            print ' <link href="' .
+            print ' <link type="text/css" href="' .
               $theme_webprefix .
               '/unauthenticated/css/palettes/' .
               (theme_night_mode() ? 'gunmetal' : lc($theme_config{'settings_navigation_color'})) . '.' .
@@ -390,7 +395,7 @@ sub embed_header
                     next;
                 }
 
-                print ' <script src="' .
+                print ' <script type="application/javascript" src="' .
                   $theme_webprefix .
                   '/unauthenticated/js/' .
                   $js .
@@ -474,7 +479,8 @@ sub embed_settings
         $admin_def_config_file = read_file_contents($admin_def_config_file);
         $admin_def_config_file =~ tr/\r\n/;/d;
         $admin_def_config_file =~ s/\s*(.*?=)'([\d\.]+)'(;)\s*/$1$2$3/g;
-        print ' <script>' . $admin_def_config_file . '</script>' . "\n";
+        print ' <script type="application/javascript">' .
+            $admin_def_config_file . '</script>' . "\n";
     }
 
     # Embed global configuration
@@ -482,7 +488,8 @@ sub embed_settings
         $global_config_file = read_file_contents($global_config_file);
         $global_config_file =~ tr/\r\n/;/d;
         $global_config_file =~ s/\s*(.*?=)'([\d\.]+)'(;)\s*/$1$2$3/g;
-        print ' <script>' . $global_config_file . '</script>' . "\n";
+        print ' <script type="application/javascript">' .
+            $global_config_file . '</script>' . "\n";
     }
 
     # Embed user configuration
@@ -490,19 +497,21 @@ sub embed_settings
         $user_config_file = read_file_contents($user_config_file);
         $user_config_file =~ tr/\r\n/;/d;
         $user_config_file =~ s/\s*(.*?=)'([\d\.]+)'(;)\s*/$1$2$3/g;
-        print ' <script>' . $user_config_file . '</script>' . "\n";
+        print ' <script type="application/javascript">' .
+            $user_config_file . '</script>' . "\n";
     }
 }
 
 sub embed_tconfig
 {
-    print ' <script>tconfig_beta_updates=' . ($tconfig{'beta_updates'} ne '1' ? 0 : 1) . '</script>' . "\n";
+    print ' <script type="application/javascript">tconfig_beta_updates=' .
+        ($tconfig{'beta_updates'} ne '1' ? 0 : 1) . '</script>' . "\n";
 }
 
 sub embed_styles
 {
     if ($theme_config{'settings_contrast_mode'} eq 'true') {
-        print ' <link href="' .
+        print ' <link type="text/css" href="' .
           $theme_webprefix .
           '/unauthenticated/css/high-contrast.' .
           (theme_debug_mode() ? 'src' : 'min') . '.css?' .
@@ -512,7 +521,7 @@ sub embed_styles
 
     my $css = $config_directory . "/$current_theme/styles.css";
     if (-r $css && -s $css) {
-        print ' <link data-custom-style href="data:text/css;base64,' .
+        print ' <link type="text/css" data-custom-style href="data:text/css;base64,' .
           trim(encode_base64(read_file_contents($css))) . '" rel="stylesheet">' . "\n";
     }
 
@@ -565,7 +574,7 @@ sub embed_pm_scripts
 sub embed_css_fonts
 {
     my ($return) = @_;
-    my $font_link = ' <link href="' .
+    my $font_link = ' <link type="text/css" href="' .
       $theme_webprefix .
       '/unauthenticated/css/fonts-roboto.' .
       (theme_debug_mode() ? 'src' : 'min') . '.css?' .
@@ -582,7 +591,7 @@ sub embed_css_unbundled
 {
     load_devel_dependencies();
     foreach my $css (@theme_bundle_css) {
-        printf " <link href=\"%s/unauthenticated/css/%s.src.css?%s\" rel=\"stylesheet\">\n",
+        printf " <link type='text/css' href=\"%s/unauthenticated/css/%s.src.css?%s\" rel=\"stylesheet\">\n",
                $theme_webprefix,
                $css,
                theme_version('timestamped');
@@ -592,7 +601,7 @@ sub embed_css_unbundled
 
 sub embed_css_bundle
 {
-    print ' <link href="' .
+    print ' <link type="text/css" href="' .
       $theme_webprefix .
       '/unauthenticated/css/bundle.min.css?' .
       theme_version('timestamped') .
@@ -602,8 +611,9 @@ sub embed_css_bundle
 
 sub embed_css_night_rider
 {
-    if (theme_night_mode_login() || theme_night_mode()) {
-        print ' <link href="' .
+    my $allow_auto = shift;
+    if (theme_night_mode_login() || theme_night_mode() || $allow_auto) {
+        print ' <link type="text/css" href="' .
           $theme_webprefix .
           '/unauthenticated/css/palettes/nightrider.' .
           (theme_debug_mode() ? 'src' : 'min') . '.css?' .
@@ -614,7 +624,7 @@ sub embed_css_night_rider
 
 sub embed_js_bundle
 {
-    print ' <script src="' .
+    print ' <script type="application/javascript" src="' .
       $theme_webprefix .
       '/unauthenticated/js/bundle.min.js?' .
       theme_version('timestamped') .
@@ -629,7 +639,8 @@ sub embed_js_scripts
     my $js = $config_directory . "/$current_theme/scripts.js";
     if (-r $js && -s $js) {
         $js = read_file_contents($js);
-        print ' <script data-custom-script>' . $js . '</script>' . "\n";
+        print ' <script type="application/javascript" data-custom-script>' .
+            $js . '</script>' . "\n";
     }
 }
 
@@ -762,7 +773,7 @@ sub embed_footer
         if (get_module_name() =~ /mysql/ ||
             get_module_name() =~ /postgresql/)
         {
-            print ' <script src="' .
+            print ' <script type="application/javascript" src="' .
               $theme_webprefix .
               '/extensions/sql.' .
               ($args[0] ? 'src' : 'min') . '.js?' .
@@ -774,7 +785,7 @@ sub embed_footer
         if (get_module_name() =~ /file-manager/ ||
             get_module_name() =~ /filemin/)
         {
-            print ' <script src="' .
+            print ' <script type="application/javascript" src="' .
               $theme_webprefix .
               '/extensions/file-manager/file-manager.' .
               ($args[0] ? 'src' : 'min') . '.js?' .
@@ -1021,14 +1032,19 @@ sub get_button_style
 
     if (string_contains($keys, 'mail_fchange')) {
         $class = "default";
+    } elsif (string_contains($keys, "cron_save")) {
+        $icon  = "clock";
+        $class = "success ";
     } elsif (string_contains($keys, 'sform_ok')) {
         $icon  = "search";
-        $class = "info";
+        # $class = "info";
     } elsif (string_contains($keys, 'edit_createnow') || string_contains($keys, 'edit_savenow')) {
         $icon = "backup fa-1_25x";
     } elsif (string_contains($keys, "pass_ok")) {
         $icon  = " fa2 fa2-key";
         $class = "warning ";
+    } elsif (string_contains($keys, "feat_backup_refresh")) {
+        $icon  = "refresh-mdi fa-1_25x margined-left--3";
     } elsif (string_ends_with($keys, "_gnupg") ||
              string_contains($keys, 'secret_setup') ||
              string_contains($keys, 'ssl_gen')      ||
@@ -1137,6 +1153,9 @@ sub get_button_style
     } elsif (string_contains($keys, "ifaces_apply")) {
         $class = "transparent-force ";
         $icon  = "toggle-switch  fa-1_05x";
+    } elsif ($keys eq 'scripts_kit_apply') {
+        $class = "info ";
+        $icon  = " fa2 fa2-marker-check fa-0_95x margined-top-1";
     } elsif (string_contains($keys, "apply")) {
         $class = "info ";
         $icon  = "check-circle-o";
@@ -1307,7 +1326,13 @@ sub get_button_style
         $icon = "clear-all fa-1_25x";
     } elsif (string_contains($keys, "history_ok")) {
         $icon = "area-chart";
-    } elsif (string_contains($keys, "edit_open") || string_contains($keys, "edit_list")) {
+    } elsif ($keys eq "feat_backup_create") {
+        $icon = "plus-circle margined-right-3";
+        $class = "success ";
+    } elsif ($keys eq "feat_backup_backuplog") {
+        $icon = "backup fa-1_15x margined-left--1 margined-right-1";
+    } elsif (string_contains($keys, "edit_open") ||
+             string_contains($keys, "edit_list")) {
         $icon = "files-o";
     } elsif (string_contains($keys, "reboot") ||
              string_contains($keys, "view_refresh")  ||
@@ -1338,7 +1363,7 @@ sub get_button_style
              string_contains($keys, "scripts_findok") ||
              string_contains($keys, "kill_title"))
     {
-        $class = "info ";
+        # $class = "info ";
         $icon  = "search";
     } elsif (string_contains($keys, "dmass_move") ||
              string_contains($keys, "domains_move") ||
@@ -1387,9 +1412,6 @@ sub get_button_style
         $icon = "stop";
     } elsif (string_contains($keys, "index_script")) {
         $icon = "update";
-    } elsif (string_contains($keys, "cron_ok")) {
-        $icon  = "check-circle-o";
-        $class = "success ";
     } elsif (string_contains($keys, "status")) {
         $icon = "info-circle";
     } elsif (string_contains($keys, "warnok")) {
@@ -1469,10 +1491,15 @@ sub get_button_style
         $icon  = "rotate-3d fa-1_25x margined-left--3 margined-right--3";
     } elsif (string_contains($keys, "csv")) {
         $icon = "export";
-    } elsif (string_contains($keys, "restore")) {
+    } elsif (string_contains($keys, "restore") ||
+             $keys eq "feat_backuplog") {
         $icon = "restore fa-1_25x";
         if (string_contains($keys, "restore_now2")) {
             $class = "success ";
+        } elsif (string_contains($keys, "quick_restore")) {
+            $class = "info ";
+        } elsif ($keys eq "feat_backuplog") {
+            $class = "grey ";
         }
     } elsif (string_contains($keys, "backup_title") ||
              string_contains($keys, "dbase_backup") ||
@@ -1482,6 +1509,9 @@ sub get_button_style
              string_contains($keys, "backup_now"))
     {
         $icon = "backup fa-1_25x";
+        if ($keys eq "feat_backup_now") {
+            $class = "info ";
+        }
     } elsif (string_contains($keys, "dbase_exec") ||
              string_contains($keys, "exec_exec")         ||
              string_contains($keys, "user_priv_execute") ||
@@ -1494,7 +1524,10 @@ sub get_button_style
              string_contains($keys, "view_title1"))
     {
         $icon = "list";
-    } elsif (string_contains($keys, "table_data")) {
+    } elsif (string_contains($keys, "delnow")) {
+        $icon = "trash fa-1_05x";
+        $class = "danger ";
+    }  elsif (string_contains($keys, "table_data")) {
         $icon = "database-outline";
     } elsif (string_contains($keys, "index_title1") || string_contains($keys, "table_index")) {
         $icon = "key-plus fa-1_25x";
@@ -1574,6 +1607,24 @@ sub get_button_style
     return ($keys, $class, $icon);
 }
 
+# Special combined buttons labels depending on the currently selected tab
+# within nested forms
+sub filter_label_compound
+{
+    my ($label, $key) = @_;
+    my ($base) = $key =~ /^(.*?)(?:_n\d+)?$/;
+    my @out;
+    for my $i (1 .. 5) {
+        my $part_key = "${base}_n$i";
+        my $part     = text($part_key);
+        last if !defined $part || $part eq $part_key;
+        $part = &html_escape($part);
+        push @out, qq(<span data-compound="$i">$part</span>);
+    }
+    my $rs = @out > 1 ? join('', @out) : &html_escape($label);
+    return $rs."&nbsp;";
+}
+
 sub embed_login_head
 {
     my ($inline) = @_;
@@ -1586,30 +1637,32 @@ sub embed_login_head
     print ' <meta name="color-scheme" content="only light">', "\n";
     embed_noscript();
     print ' <meta charset="utf-8">', "\n";
-    print ' <script type="text/javascript">try{const e=new URL(location.href),a=e.pathname,s=/\/(session_login|pam_login)\.cgi$/.test(a),r=e.searchParams.has("logout"),t=e.searchParams.has("returned-username")&&/^\S+$/.test(e.searchParams.get("returned-username"));(s&&r||t)&&(e.pathname=a.substring(0,a.lastIndexOf("/")),e.searchParams.delete("logout"),e.searchParams.delete("returned-username"),history.pushState({},"",e))}catch(e){}</script>', "\n";
+    print ' <script type="application/javascript">try{const e=new URL(location.href),a=e.pathname,s=/\/(session_login|pam_login)\.cgi$/.test(a),t=e.searchParams.has("logout"),r=e.searchParams.has("returned-username")&&/^\S+$/.test(e.searchParams.get("returned-username"));(s&&t||r)&&(e.pathname=a.substring(0,a.lastIndexOf("/")),e.pathname.endsWith("/")||(e.pathname+="/"),e.searchParams.delete("logout"),e.searchParams.delete("returned-username"),history.pushState({},"",e))}catch(e){}</script>', "\n";
     embed_favicon('login-page');
     print ' <title>', $title, '</title>', "\n";
     print ' <meta name="viewport" content="width=device-width, initial-scale=1.0">' . "\n";
     # Print object with language strings
     if ($gconfig{'forgot_pass'}) {
-        print ' <script>';
+        print ' <script type="application/javascript">', "\n";
         print 'const theme_language = ' . get_theme_language_login();
         print "</script>\n";
     }
+    my $theme_night_mode_login_auto = theme_night_mode_login_auto();
     if ($inline) {
         my $file_contents = read_file_contents("$root_directory/$current_theme/unauthenticated/css/bundle.min.css");
         print '<style>';
         print $file_contents;
         print '</style>';
 
-        if (theme_night_mode_login()) {
+        if (theme_night_mode_login(1)) {
             my $file_contents =
               read_file_contents("$root_directory/$current_theme/unauthenticated/css/palettes/nightrider.min.css");
             print '<style>';
             print $file_contents;
             print '</style>';
+            print $theme_night_mode_login_auto if ($theme_night_mode_login_auto);
         }
-        my $file_contents = read_file_contents("$root_directory/$current_theme/unauthenticated/css/fonts-roboto.min.css");
+        $file_contents = read_file_contents("$root_directory/$current_theme/unauthenticated/css/fonts-roboto.min.css");
         print '<style>';
         print $file_contents;
         print '</style>';
@@ -1621,20 +1674,23 @@ sub embed_login_head
             embed_css_bundle();
         }
 
-        print ' <script src="' .
+        print ' <script type="application/javascript" src="' .
               $theme_webprefix .
               "/unauthenticated/js/session-login.$ext.js?" .
               theme_version('timestamped') .
               '"></script>' . "\n";
         
-        embed_css_night_rider();
+        embed_css_night_rider(1);
+        if ($theme_night_mode_login_auto) {
+            print $theme_night_mode_login_auto;
+        }
     }
 
     embed_background();
     embed_styles();
     embed_overlay_head();
     if (get_env('script_name') =~ /password_change\.cgi$/) {
-        print '<script>';
+        print '<script type="application/javascript">';
         print 'document.addEventListener("DOMContentLoaded", function() {';
         my $palette = theme_night_mode_login() ? 'nightRider' : 'gainsboro';
             print 'document.querySelector("html").setAttribute("data-bgs", "' . $palette . '");';
@@ -1654,6 +1710,30 @@ sub theme_night_mode
     }
 }
 
+sub theme_night_mode_login_auto
+{
+    if ($theme_config{'settings_global_palette_unauthenticated'} eq 'auto') {
+        my $detect;
+        $detect = '<script type="application/javascript">console.log("Auto dark mode");';
+        $detect .= 'if (window.matchMedia && window.matchMedia("(prefers-color-scheme: dark)").matches) {';
+        $detect .= 'document.querySelector("html").setAttribute("data-bgs", "nightRider")';
+        $detect .= '} else {';
+        $detect .= 'document.querySelector("html").setAttribute("data-bgs", "gainsboro");';
+        $detect .= '}';
+        $detect .= 'window.matchMedia("(prefers-color-scheme: dark)").addEventListener("change", function(e) {';
+        $detect .= 'if(e.matches) {';
+        $detect .= 'document.querySelector("html").setAttribute("data-bgs", "nightRider");';
+        $detect .= '} else {';
+        $detect .= 'document.querySelector("html").setAttribute("data-bgs", "gainsboro");';
+        $detect .= '}';
+        $detect .= '});';
+        $detect .= '</script>';
+        return $detect;
+    } else {
+        return 0;
+    }
+}
+
 sub theme_night_mode_login
 {
     if ($theme_config{'settings_global_palette_unauthenticated'} eq 'dark') {
@@ -1735,10 +1815,22 @@ sub theme_post_update
     }
 }
 
+sub theme_parse_proxy_redirect
+{
+my ($in, $env) = @_;
+if ($env->{'HTTP_REFERER'} && $env->{'HTTP_X_PJAX_URL'} &&
+    $env->{'HTTP_X_PJAX_URL'} =~ /(\S*?\/link\.cgi\/[\d]{8,16})/ && 
+    $in->{'proxy-redirect'} =~ /\S+/) {
+    return $in->{'proxy-redirect'};
+}
+return undef;
+}
+
 sub header_html_data
 {
     my ($module, $skip, @args) = @_;
-    my $redirect = get_theme_temp_data('redirected');
+    my $redirect = theme_parse_proxy_redirect(\%in, \%ENV) ||
+                   get_theme_temp_data('redirected');
     $redirect = "" if ($redirect eq "/");
     return 'data-redirect="' . $redirect .
 
@@ -1792,6 +1884,8 @@ sub header_html_data
       ($skip ? '' : ' data-default-theme="' . $theme_config{'settings_navigation_color'} . '"') .
       ' data-editor-palette="' .
       $theme_config{'settings_cm_editor_palette'} .
+      '" data-static-theme-version="' .
+      theme_version('version') .
       '" data-theme-version="' .
       theme_version('version') .
       '" data-theme-version-data="' .
@@ -2222,7 +2316,7 @@ sub embed_product_branding
     }
     $brand =
 "<div tabindex=\"1\" class=\"branding-backdrop $brand_name\"><div class=\"centered\">$brand<br><div class=\"branding-loader\">$loader</div></div></div>";
-    $brand .= "<script>page.branding.process()</script>";
+    $brand .= "<script type='application/javascript'>page.branding.process()</script>";
     print $brand;
 }
 
