@@ -680,7 +680,7 @@ sub theme_ui_form_end
 
         # When going back to a form, re-enable any text fields generated by
         # ui_opt_textbox that aren't in the default state.
-        $rv .= "<script>\n";
+        $rv .= "<script type='application/javascript'>\n";
         $rv .= "var opts = document.getElementsByClassName('ui_opt_textbox');\n";
         $rv .= "for(var i=0; i<opts.length; i++) {\n";
         $rv .= "  opts[i].disabled = document.getElementsByName(opts[i].name+'_def')[0].checked;\n";
@@ -909,7 +909,7 @@ sub theme_ui_textarea
 
 sub theme_ui_submit
 {
-    my ($label, $name, $dis, $tags, $icon_class, $btn_class_extra) = @_;
+    my ($label, $name, $dis, $tags, $icon_class, $btn_class_extra, $text_key) = @_;
     my ($keys, $class, $icon) = get_button_style($label);
     if ($icon_class && !$icon) {
         $icon = "<i class=\"$icon_class\"></i>";
@@ -918,8 +918,12 @@ sub theme_ui_submit
     $ids = "_s_$main::ui_submit_tcalled" if ($main::ui_submit_tcalled++);
 
     my $nbsp;
+    my $clabel;
     if ($label) {
         $nbsp = "&nbsp;";
+        if ($text_key && $text_key =~ /_compound_\w+/) {
+            $clabel = filter_label_compound($label, $text_key);
+        }
     }
     # Form attr has to be passed directly to input
     my $form_value = "";
@@ -932,7 +936,7 @@ sub theme_ui_submit
       ($name ne '' ? " name=\"" . &quote_escape($name) . "\""      : "") .
       ($name ne '' ? " id=\"" . &quote_escape($name . $ids) . "\"" : "") .
       ($dis        ? " disabled=true" : "") . ($tags ? " " . $tags : "") . ">" . $icon . "$nbsp<span data-entry=\"$keys\">" .
-      &html_escape($label) . "$nbsp</span></button>\n" . "<input$form_value class=\"hidden\" type=\"submit\""
+      ($clabel || (&html_escape($label) . "$nbsp"))."</span></button>\n" . "<input$form_value class=\"hidden\" type=\"submit\""
       .
       ( $name ne '' ? " name=\"" . &quote_escape($name) . "\" value=\"" . &quote_escape($label) . "\"" :
           ""
@@ -1223,7 +1227,7 @@ sub theme_ui_opt_textbox
 
     # Optional (extra) elements must also be disabled (or enabled) on load
     if (defined($extra)) {
-        $rv .= "<script>\$('#content form input[name=\"${name}_def\"]:checked').click();</script>";
+        $rv .= "<script type='application/javascript'>\$('#content form input[name=\"${name}_def\"]:checked').click();</script>";
     }
     return $rv;
 }
@@ -1349,11 +1353,25 @@ sub theme_ui_buttons_row
     if (ref($hiddens)) {
         $hiddens = join("\n", map {&ui_hidden(@$_)} @$hiddens);
     }
-    return
-      "<tr data-ui-buttons-row-form-container>\n<td>\n<form action='$script' method='post' class='ui_buttons_form'>\n" .
-      $hiddens . "<table>" . "<tr class='ui_buttons_row'> " . "<td data-nowrap class=ui_buttons_label>" .
-      ($before ? $before . " " : "") . &ui_submit($label) . ($after ? " " . $after : "") . "</td>\n" .
-      "<td class=ui_buttons_value><span>" . $desc . "</span></td></tr>\n" . "</table>\n" . "</form>\n</td>\n</tr>\n";
+    my $btn = ($before ? "$before " : "")
+            . &ui_submit($label)
+            . ($after  ? " $after"  : "");
+    
+    return <<"HTML";
+<tr data-ui-buttons-row-form-container>
+    <td>
+    <form action='$script' method='get' class='ui_buttons_form'>
+        $hiddens
+        <table>
+        <tr class='ui_buttons_row'>
+            <td data-nowrap class='ui_buttons_label'>$btn</td>
+            <td class='ui_buttons_value'><span>$desc</span></td>
+        </tr>
+        </table>
+    </form>
+    </td>
+</tr>
+HTML
 }
 
 sub theme_ui_buttons_end
@@ -1421,6 +1439,13 @@ sub theme_redirect
     my ($arg1, $arg2) = (navigation_link_clean($_[0]),
                          navigation_link_clean($_[1]));
 
+    # Bail out here if the proxy redirect is enforced
+    my ($url_ready, $no_proxy_redirect) = theme_no_proxy_url($_[1]);
+    if ($no_proxy_redirect && $url_ready) {
+        print "Location: $url_ready\n\n";
+        return;
+    }
+
     # Clean redirected links query string if requested
     if ($request_uri =~ /no-query=string/) {
         my $nocache = "no-cache=1";
@@ -1468,6 +1493,19 @@ sub theme_redirect
     }
 }
 
+sub theme_no_proxy_url
+{
+my ($url) = @_;
+if ($url && $url =~ /proxy-redirect=(\S+)/) {
+        $url =~ s{/[^/]+/link\.cgi/\d+/}{/}g;
+        $url =~ s{[?&]proxy-redirect=\d+(?:&|$)}{};
+        $url =~ s/[?&]$//;
+        $url =~ s{^(?:[a-z][a-z0-9+.-]*:)?//[^/]*}{}i;
+        return ($url, 1);
+        }
+return ($url, 0);
+}
+
 sub theme_redirect_url_alterer
 {
     my ($u) = @_;
@@ -1510,7 +1548,7 @@ sub theme_header_redirect_download
     print "</head>\n";
     my $script =
       '<form data-predownload action="' .
-      $url . '" method="post" name="redirect"></form><script>setTimeout(function(){document.forms.redirect.submit()}, ' .
+      $url . '" method="post" name="redirect"></form><script type="application/javascript">setTimeout(function(){document.forms.redirect.submit()}, ' .
       ($delay ? $delay . "000" : 0) . ');</script>';
     print "<body>\n";
     print $script . "\n";
@@ -1557,7 +1595,7 @@ sub theme_js_redirect
         # If the URL is like /foo , add webprefix
         $url = &get_webprefix().$url;
     }
-    return "<script type='text/javascript'>"."
+    return "<script type='application/javascript'>"."
             try {
                 progress.end();
                 set_onbeforeunload_status_native(0);
@@ -1570,14 +1608,14 @@ sub theme_js_redirect
 sub theme_post_save_domain
 {
     my ($d) = @_;
-    print '<script>';
+    print '<script type="application/javascript">';
     print 'theme_post_save=' . ($d->{'id'} ? $d->{'id'} : '-1') . '', "\n";
     print "</script>\n";
 }
 
 sub theme_post_save_domains
 {
-    print '<script>';
+    print '<script type="application/javascript">';
     print 'theme_post_save=-1', "\n";
     print "</script>\n";
 }
@@ -1589,7 +1627,7 @@ sub theme_post_save_server
         $action eq 'delete' ||
         !$done_theme_post_save_server++)
     {
-        print '<script>';
+        print '<script type="application/javascript">';
         print 'theme_post_save=' . ($s->{'id'} ? $s->{'id'} : '-1') . '', "\n";
         print "</script>\n";
     }
@@ -1598,7 +1636,7 @@ sub theme_post_save_server
 sub theme_select_domain
 {
     my ($s) = @_;
-    print '<script>';
+    print '<script type="application/javascript">';
     print 'theme_select_server=' . ($s->{'id'} ? $s->{'id'} : '') . '', "\n";
     print "</script>\n";
 }
@@ -1606,7 +1644,7 @@ sub theme_select_domain
 sub theme_select_server
 {
     my ($s) = @_;
-    print '<script>';
+    print '<script type="application/javascript">';
     print 'theme_select_server=' . ($s->{'id'} ? $s->{'id'} : '0') . '', "\n";
     print "</script>\n";
 }
@@ -1620,7 +1658,7 @@ sub theme_post_change_theme
     error_40x_handler(1);
     
     # Remove service worker
-    print '<script>';
+    print '<script type="application/javascript">';
     print 'navigator.serviceWorker.getRegistrations().then(
         registrations => {registrations.forEach(reg => reg.unregister());});';
     print "</script>\n";
@@ -1628,7 +1666,7 @@ sub theme_post_change_theme
 
 sub theme_post_change_modules
 {
-    print '<script>';
+    print '<script type="application/javascript">';
     print 'theme_post_save=-1', "\n";
     print "</script>\n";
 }
