@@ -270,6 +270,18 @@ sub nav_detector
     $tab  = $nav_def_tab;
     $page = $page;
 
+    # Do we have navigation goto request for a module with its own switch?
+    if ($page) {
+        my ($nmod) = $page =~ /[?&]nmod=([0-9a-zA-Z-]+)/;
+        if (($nmod eq $mod_vm || $nmod eq $mod_cm) && foreign_available($nmod)) {
+            if ($nmod eq $mod_vm) {
+                $tab = $prd_vm;            # Override to use Virtualmin tab
+            } elsif ($nmod eq $mod_cm) {
+                $tab = $prd_cm;            # Override to use Cloudmin tab
+            }
+        }
+    }
+
     return ($tab, $page);
 }
 
@@ -340,7 +352,7 @@ sub nav_virtualmin_menu
     $rv .= $menu->{'after'}
       if ($menu->{'after'});
     $rv .= nav_theme_links();
-    $rv .= nav_links($menu->{'mode'});
+    $rv .= nav_links($menu->{'mode'}, $mod);
     $rv .= nav_menu_html_snippet();
     $rv .= nav_detect_page($page, 1);
     $rv .= nav_detect_script(1);
@@ -357,7 +369,7 @@ sub nav_cloudmin_menu
     my $rv     = $menu->{'before'};
     $rv .= nav_link_sysinfo(undef, $mod);
     $rv .= nav_theme_links();
-    $rv .= nav_links($menu->{'mode'});
+    $rv .= nav_links($menu->{'mode'}, $mod);
     $rv .= nav_menu_html_snippet();
     $rv .= nav_detect_page($page);
     $rv .= nav_detect_script();
@@ -401,6 +413,50 @@ sub nav_menu
     } else {
         $rv = nav_webmin_menu($page);
     }
+    if ($page =~ m{/link\.cgi}) {
+        # X10101
+        $rv .= <<'SCRIPT';
+<script type="application/javascript">
+v___target_servers_index_loading = 1;
+(function () {
+    const loadedVersion = function() {
+        const m = document.querySelector('link[href*="fonts-roboto"]')?.href.match(/\?(\d{2})/);
+        return m ? parseInt(m[1], 10) : 25;
+    };
+    if (loadedVersion() < 25 || !vars.navigation.reload_timeout) {
+        return;
+    }
+    if (window.v___initial_load) {
+        setTimeout(function () {
+            let tries = 0;
+            const maxTries = 2e4;
+            const trying = function (tries_exceeded) {
+                if (
+                    tries_exceeded ||
+                    (location.href.indexOf("link.cgi") !== -1 &&
+                     !$("body").hasClass("loading-bg"))
+                ) {
+                    clearInterval(timer);
+                    plugins.navigation.reload(-1, function () {
+                        plugins.navigation.sync(function () {
+                            plugins.navigation.load.initializing = 0;
+                            plugins.navigation.load.stop();
+                        });
+                    });
+                } else if (++tries >= maxTries) {
+                    clearInterval(timer);
+                    trying(true);
+                }
+            };
+            const timer = setInterval(trying, 10);
+        }, vars.navigation.reload_timeout);
+        plugins.navigation.load.initializing = 1;
+        plugins.navigation.load.start();
+    }
+})();
+</script>
+SCRIPT
+    }
     return $rv;
 }
 
@@ -449,7 +505,7 @@ sub nav_detect_page
         $page =~ s/&amp;/&/g;
         my $last_id = $set_last ? nav_set_last_id($page) : undef;
         if ($last_id) {
-            $last_id = "<script>$last_id</script>";
+            $last_id = "<script type='application/javascript'>$last_id</script>";
         }
         $rv .= "<li data-goto=\"$page\" class=\"hidden\">$last_id</li>\n";
     }
@@ -702,7 +758,7 @@ sub nav_list_combined_menu
     my $search_bar = '';
 
     my $gwp = sub {
-        my ($link) = @_;
+        my ($link, $mod) = @_;
 
         # Link could but shouldn't end with just &
         $link =~ s/&amp;$//;
@@ -711,11 +767,17 @@ sub nav_list_combined_menu
             if (!string_starts_with($link, "http") &&
                 !string_starts_with($link, "ftp") &&
                 !string_starts_with($link, "www") &&
-                !string_starts_with($link, "../"))
+                !string_starts_with($link, "../") &&
+                !string_contains($link, "config.cgi"))
             {
                 $link = "/$link" if (!string_starts_with($link, "/"));
                 $link = "$theme_webprefix$link"
                   if ($link !~ /^\Q$theme_webprefix\E/);
+                # Add module reference to all non module links
+                if ($mod && $link !~ /$mod\// && $link !~ /[?&]nmod=/) {
+                    my $sep = ($link =~ /\?/) ? '&' : '?';
+                    $link .= "${sep}nmod=$mod";
+                }
             }
         }
         return $link;
@@ -739,11 +801,11 @@ sub nav_list_combined_menu
         }
         $search_bar = nav_search($vm_new_format);
     }
-
+    my $known_module = $modules ? $modules->[-1] : undef;
     foreach my $item (@$items) {
         if ((grep {$_ eq $item->{'module'}} @{$modules}) || $group) {
 
-            my $link = &$gwp($item->{'link'});
+            my $link = &$gwp($item->{'link'}, $item->{'module'} || $known_module);
             my $icon;
             my $rv_after_local;
 
@@ -758,7 +820,7 @@ sub nav_list_combined_menu
                     $icon = '<i class="fa fa-fw fa-tasks"></i>';
                 } elsif ($link =~ /\/virtualmin-wp-workbench/)
                 {
-                    $icon = '<i class="fa fa-fw fa2 fa2-workbench margined-left--05 margined-right-05"></i>';
+                    $icon = '<i class="fa fa-fw fa2 fa2-workbench margined-left--1 margined-right-1"></i>';
                 } elsif ($link =~ /\/virtual-server\/edit_newvalidate\.cgi/ &&
                          !&webmin_user_is_admin())
                 {
@@ -1002,7 +1064,7 @@ sub nav_list_combined_menu
 # Returns a list of tiny square buttons for navigation menu
 sub nav_links
 {
-    my ($login_mode) = @_;
+    my ($login_mode, $mod) = @_;
     my $rv;
     $rv =
       '<li class="menu-container"><ul class="user-links"><li data-collapse-trigger-container data-linked' .
@@ -1015,11 +1077,12 @@ sub nav_links
     if ($theme_config{'settings_sysinfo_link_mini'} eq 'true' &&
         dashboard_switch() ne '1')
     {
+        $mod = "?mod=$mod" if ($mod);
         $rv .= '<li data-linked' .
           get_button_tooltip('theme_xhred_titles_dashboard', 'settings_hotkey_sysinfo', 'auto top') . ' class="user-link">';
         $rv .=
           '<a tabindex="-1" class="menu-exclude-link sidebar_sysinfo_link" href="' .
-          $theme_webprefix . '/sysinfo.cgi"><i class="fa fa-fw fa-' .
+          $theme_webprefix . '/sysinfo.cgi'.$mod.'"><i class="fa fa-fw fa-' .
           ($get_user_level eq '3' ? 'user-circle' : 'dashboard') . '"></i></a>';
         $rv .= '</li>';
     }
@@ -1033,7 +1096,15 @@ sub nav_links
           ' hidden'
       ) .
       '">';
-    $rv .= '<span><i class="fa fa-fw ' . (theme_night_mode() ? 'fa-sun' : 'fa-moon') . '"></i></span>';
+    $rv .= '<span><i class="fa fa-fw ' .
+        ($theme_config{'settings_palette_auto'} eq 'true'
+            ? ('fa-auto-palette' . (theme_night_mode()
+                ? ' sun-icon'
+                : ' moon-icon'))
+            : theme_night_mode()
+                ? 'fa-sun sun-icon'
+                : 'fa-moon moon-icon'
+        ) . '"></i></span>';
     $rv .= '</li>';
 
     if ((foreign_available("xterm") && $theme_config{'settings_show_terminal_link'} ne 'false') ||
