@@ -31,10 +31,11 @@ sub theme_settings_raw
                        'settings_right_virtualmin_default',
                        'settings_right_cloudmin_default',
                        'settings_right_page_keep',
-                       'settings_right_reload',
+                       'settings_right_sync',
                        'settings_document_title',
-                       'settings_cm_editor_palette',
+                       'settings_palette_auto',
                        'settings_global_palette_unauthenticated',
+                       'settings_cm_editor_palette',
                        'settings_theme_config_admins_only_privileged',
                        'settings_embed_product_splash_privileged',
             ] }
@@ -107,6 +108,7 @@ sub theme_settings_raw
             'title' => &theme_text('settings_right_table_options'),
             'data'  => [
                        'settings_right_table_links_type',
+                       'settings_right_table_links_sorted',
                        'settings_right_table_animate_icons',
                        'settings_right_table_grayscaled_icons',
                        'settings_right_table_layout_control',
@@ -180,7 +182,7 @@ sub theme_settings_filter
     {
         push(@theme_settings_filter,
              'settings_right_default_tab_webmin',
-             'settings_right_reload', 'settings_right_page_keep');
+             'settings_right_sync', 'settings_right_page_keep');
     }
 
     # Assign default for options provided by the server side
@@ -263,6 +265,7 @@ sub theme_settings_filter
              'settings_sysinfo_easypie_charts_scale',
              'settings_sysinfo_max_servers',
              'settings_sysinfo_real_time_status',
+             'settings_sysinfo_real_time_stored_duration',
              'settings_leftmenu_section_hide_refresh_modules',
              'settings_leftmenu_section_hide_unused_modules',
              'settings_leftmenu_netdata',
@@ -309,7 +312,7 @@ sub theme_settings_format
         }
     }
 
-    my $v = (length $theme_config{$k} ? $theme_config{$k} : $v);
+    $v = (length $theme_config{$k} ? $theme_config{$k} : $v);
 
     if ($v eq 'true' || $v eq 'false') {
         $v = ui_yesno_radio($k, $v, 'true', 'false');
@@ -554,18 +557,43 @@ sub theme_settings_format
 
     } elsif ($k eq 'settings_navigation_color') {
         $v = settings_get_select_navigation_color($v, $k);
-    } elsif ($k eq 'settings_cm_editor_palette') {
-        $v = settings_get_select_editor_color($v, $k);
     } elsif ($k eq 'settings_global_palette_unauthenticated') {
         $v = ui_select($k,
                        $v,
-                       [[('light', $theme_text{'theme_xhred_global_light'})],
+                       [[('auto', $theme_text{'theme_xhred_global_auto'})],
+                        [('light', $theme_text{'theme_xhred_global_light'})],
                         [('dark',  $theme_text{'theme_xhred_global_dark'})]
                        ]);
+    } elsif ($k eq 'settings_cm_editor_palette') {
+        $v = settings_get_select_editor_color($v, $k);
+    } elsif ($k eq 'settings_right_sync') {
+        $v = ui_select($k, $v, [
+            [2, $theme_text{"settings_right_sync2"}],
+            [1, $theme_text{"settings_right_sync1"}],
+            [0, $theme_text{"settings_right_sync0"}]
+        ]);
     } elsif ($k eq 'settings_document_title') {
         $v = settings_get_select_document_title($v, $k);
     } elsif ($k eq 'settings_sysinfo_real_time_status') {
-        $v = ui_radio($k, $v, [[1, $text{'yes'}], [0, $text{'no'}]]);
+        my $realtime = 'settings_sysinfo_real_time_';
+        my $realtime_pref = "${realtime}status_";
+        my $duration_key = "${realtime}stored_duration";
+        my $select = ui_select($duration_key, $theme_config{$duration_key},
+                       [
+                        [(900, $theme_text{"${realtime_pref}history_duration1"})],
+                        [(1800, $theme_text{"${realtime_pref}history_duration2"})],
+                        [(2700, $theme_text{"${realtime_pref}history_duration3"})],
+                        [(3600, $theme_text{"${realtime_pref}history_duration4"})],
+                        [(21600, $theme_text{"${realtime_pref}history_duration5"})],
+                        [(43200, $theme_text{"${realtime_pref}history_duration6"})],
+                        [(64800, $theme_text{"${realtime_pref}history_duration7"})],
+                        [(86400, $theme_text{"${realtime_pref}history_duration8"})]
+                       ]);
+        $v = ui_radio($k, $v, [
+            [1, $theme_text{"${realtime_pref}history1"}.$select],
+            [2, $theme_text{"${realtime_pref}history2"}],
+            [0, $text{'no'}]
+        ]);
     } elsif ($k eq 'settings_right_table_links_type') {
         $v = ui_radio($k,
                       $v,
@@ -738,6 +766,9 @@ sub settings_get_select_editor_color
     my ($v, $k) = @_;
     return '<select class="ui_select" name="' . $k . '">
 
+            <option value="auto"'
+      . ($v eq 'auto' && ' selected') . '>' . $theme_text{'theme_xhred_global_auto'} . '</option>
+
             <option value="monokai"'
       . ($v eq 'monokai' && ' selected') . '>' . $theme_text{'theme_xhred_global_dark'} . '</option>
 
