@@ -6,10 +6,11 @@ var
     settings_right_virtualmin_default = 'sysinfo.cgi',
     settings_right_cloudmin_default = 'index.cgi',
     settings_right_page_keep = true,
-    settings_right_reload = true,
+    settings_right_sync = 1,
     settings_document_title = 1,
+    settings_palette_auto = true,
+    settings_global_palette_unauthenticated = 'auto',
     settings_cm_editor_palette = 'monokai',
-    settings_global_palette_unauthenticated = 'light',
     settings_theme_config_admins_only_privileged = false,
     settings_embed_product_splash_privileged = false,
 
@@ -77,13 +78,12 @@ var
     settings_hotkey_shell = 'k',
     settings_hotkey_shell2 = 'k',
     settings_hotkey_sysinfo = 'i',
-    settings_hotkey_toggle_slider = 'n',
     settings_hotkey_favorites = 'f',
     settings_hotkey_focus_search = 's',
     settings_hotkey_reload = 'r',
     settings_hotkey_navigation = 'a',
     settings_hotkey_slider = 'e',
-    settings_hotkey_toggle_key_night_mode = 'l',
+    settings_hotkey_toggle_key_night_mode = 't',
     settings_hotkey_logout_dbl = '[',
 
     // Custom links hotkeys options defaults
@@ -147,7 +147,7 @@ var
     // E.g. to change options below, you would need to open browser's console,
     // change an option value and call `theme.config.save()` function.
     //
-    settings_table_auto_datatables = true,
+    settings_table_auto_dt = true,
     settings_table_init_datatables = 20000,
     settings_right_page_hide_persistent_vscroll = true,
     settings_hide_top_loader = false,
@@ -161,7 +161,7 @@ var
     settings_leftmenu_vm_cm_dropdown_icons = true,
     settings_perform_content_scrolling = true,
     settings_sysinfo_real_time_run_rate = 1000, // 1 second tick rate (the higher the less CPU intensive it is)
-    settings_sysinfo_real_time_stored_duration = 1200, // 20 minutes of stored stats data (max: 3600; min: 300)
+    settings_sysinfo_real_time_stored_duration = 1800, // 30 minutes of stored stats data (max default: 3600; max manual: 86400-highly not recommended as will crash browser tab),
     settings_sysinfo_real_time_shutdown_on_last = false,
     settings_sysinfo_cache_timeout = 86400,
     settings_sysinfo_cpu_fans_base_rpm = 1000,
@@ -180,6 +180,9 @@ var
 
     // ConfigServer Security & Firewall related
     settings_sysinfo_csf_updates = true,
+    settings_csf_download_domain_privileged = 'download.configserver.com',
+    settings_csf_download_port_privileged = 443,
+    settings_csf_download_path_privileged = '/csf/version.txt',
     settings_sysinfo_csf_temp_list_privileged = true,
 
     // File Manager related
@@ -209,7 +212,7 @@ var
     settings_right_table_wrap_inside_scrolled_container = false,
     config_portable_table_rows_to_columns = true,
     config_portable_table_rows_to_columns_number = 2,
-    config_portable_table_rows_to_columns_width = 400,
+    config_portable_table_rows_to_columns_width = 340,
     config_portable_table_rows_to_columns_break = true,
 
     // This option can be set to any absolute or relative path
@@ -249,7 +252,8 @@ var
             {
                 'virtual-server': [
                     ['edit_newautoconfig', 'outlook thunderbird mailspring k-9 k9 mailclient autoconfig autodiscover'],
-                    ['edit_limits', 'jail chroot allowed acl shell'],
+                    ['edit_res', 'cpu memory disk process limits cgroup cgroups control groups'],
+                    ['edit_limits', 'jail cage chroot allowed acl shell isolation'],
                     ['rename_form', 'rename alter'],
                     ['edit_newmysqls', 'MySQL MariaDB'],
                     ['dkim', 'DKIM, opendkim email'],
