@@ -925,7 +925,7 @@ $table .= &ui_columns_row(\@cols, [ "valign=top", "valign=top", "valign=top",
 				    "valign=top", "valign=top" ]);
 $table .= &ui_columns_end();
 $table .= $text{'edit_ctrl'};
-$rv .= &ui_table_row(undef, $table, $width);
+$rv .= &ui_table_row(undef, $table, $width, undef, ['data-schedule-tr']);
 return $rv;
 }
 
@@ -1209,63 +1209,6 @@ foreach $w (@wds) {
 $_[0]->{'weekdays'} = join(",", @wds);
 }
 
-=head2 create_wrapper(wrapper-path, module, script)
-
-Creates a wrapper script which calls a script in some module's directory
-with the proper webmin environment variables set. This should always be used
-when setting up a cron job, instead of attempting to run a command in the
-module directory directly.
-
-The parameters are :
-
-=item wrapper-path - Full path to the wrapper to create, like /etc/webmin/yourmodule/foo.pl
-
-=item module - Module containing the real script to call.
-
-=item script - Program within that module for the wrapper to run.
-
-=cut
-sub create_wrapper
-{
-local $perl_path = &get_perl_path();
-&open_tempfile(CMD, ">$_[0]");
-&print_tempfile(CMD, <<EOF
-#!$perl_path
-open(CONF, "<$config_directory/miniserv.conf") || die "Failed to open $config_directory/miniserv.conf : \$!";
-while(<CONF>) {
-        \$root = \$1 if (/^root=(.*)/);
-        }
-close(CONF);
-\$root || die "No root= line found in $config_directory/miniserv.conf";
-\$ENV{'PERLLIB'} = "\$root";
-\$ENV{'WEBMIN_CONFIG'} = "$ENV{'WEBMIN_CONFIG'}";
-\$ENV{'WEBMIN_VAR'} = "$ENV{'WEBMIN_VAR'}";
-delete(\$ENV{'MINISERV_CONFIG'});
-EOF
-	);
-if ($gconfig{'os_type'} eq 'windows') {
-	# On windows, we need to chdir to the drive first, and use system
-	&print_tempfile(CMD, "if (\$root =~ /^([a-z]:)/i) {\n");
-	&print_tempfile(CMD, "       chdir(\"\$1\");\n");
-	&print_tempfile(CMD, "       }\n");
-	&print_tempfile(CMD, "chdir(\"\$root/$_[1]\");\n");
-	&print_tempfile(CMD, "exit(system(\"\$root/$_[1]/$_[2]\", \@ARGV));\n");
-	}
-else {
-	# Can use exec on Unix systems
-	if ($_[1]) {
-		&print_tempfile(CMD, "chdir(\"\$root/$_[1]\");\n");
-		&print_tempfile(CMD, "exec(\"\$root/$_[1]/$_[2]\", \@ARGV) || die \"Failed to run \$root/$_[1]/$_[2] : \$!\";\n");
-		}
-	else {
-		&print_tempfile(CMD, "chdir(\"\$root\");\n");
-		&print_tempfile(CMD, "exec(\"\$root/$_[2]\", \@ARGV) || die \"Failed to run \$root/$_[2] : \$!\";\n");
-		}
-	}
-&close_tempfile(CMD);
-chmod(0755, $_[0]);
-}
-
 =head2 cron_file(&job)
 
 Returns the file that a cron job is in, or will be in when it is created
@@ -1285,34 +1228,53 @@ Returns a human-readable text string describing when a cron job is run.
 =cut
 sub when_text
 {
-local $pfx = $_[1] ? "uc" : "";
-if ($_[0]->{'interval'}) {
-	return &text($pfx.'when_interval', $_[0]->{'interval'});
+my ($job, $ucfirst) = @_;
+my $pfx = $ucfirst ? "uc" : "";
+if ($job->{'interval'}) {
+	return &text($pfx.'when_interval', $job->{'interval'});
 	}
-elsif ($_[0]->{'special'}) {
+elsif ($job->{'special'}) {
 	$pfx = $_[1] ? "" : "lc";
-	return $text{$pfx.'edit_special_'.$_[0]->{'special'}};
+	return $text{$pfx.'edit_special_'.$job->{'special'}};
 	}
-elsif ($_[0]->{'boot'}) {
+elsif ($job->{'boot'}) {
 	return &text($pfx.'when_boot');
 	}
-elsif ($_[0]->{'mins'} eq '*' && $_[0]->{'hours'} eq '*' && $_[0]->{'days'} eq '*' && $_[0]->{'months'} eq '*' && $_[0]->{'weekdays'} eq '*') {
+elsif ($job->{'mins'} eq '*' && $job->{'hours'} eq '*' &&
+       $job->{'days'} eq '*' && $job->{'months'} eq '*' &&
+       $job->{'weekdays'} eq '*') {
 	return $text{$pfx.'when_min'};
 	}
-elsif ($_[0]->{'mins'} =~ /^\d+$/ && $_[0]->{'hours'} eq '*' && $_[0]->{'days'} eq '*' && $_[0]->{'months'} eq '*' && $_[0]->{'weekdays'} eq '*') {
-	return &text($pfx.'when_hour', $_[0]->{'mins'});
+elsif ($job->{'mins'} =~ /^\d+$/ && $job->{'hours'} eq '*' &&
+       $job->{'days'} eq '*' && $job->{'months'} eq '*' &&
+       $job->{'weekdays'} eq '*') {
+	return &text($pfx.'when_hour', $job->{'mins'});
 	}
-elsif ($_[0]->{'mins'} =~ /^\d+$/ && $_[0]->{'hours'} =~ /^\d+$/ && $_[0]->{'days'} eq '*' && $_[0]->{'months'} eq '*' && $_[0]->{'weekdays'} eq '*') {
-	return &text($pfx.'when_day', sprintf("%2.2d", $_[0]->{'mins'}), $_[0]->{'hours'});
+elsif ($job->{'mins'} =~ /^\d+$/ && $job->{'hours'} =~ /^\d+$/ &&
+       $job->{'days'} eq '*' && $job->{'months'} eq '*' &&
+       $job->{'weekdays'} eq '*') {
+	return &text($pfx.'when_day',
+		     sprintf("%2.2d", $job->{'mins'}),
+		     sprintf("%2.2d", $job->{'hours'}));
 	}
-elsif ($_[0]->{'mins'} =~ /^\d+$/ && $_[0]->{'hours'} =~ /^\d+$/ && $_[0]->{'days'} =~ /^\d+$/ && $_[0]->{'months'} eq '*' && $_[0]->{'weekdays'} eq '*') {
-	return &text($pfx.'when_month', sprintf("%2.2d", $_[0]->{'mins'}), $_[0]->{'hours'}, $_[0]->{'days'});
+elsif ($job->{'mins'} =~ /^\d+$/ && $job->{'hours'} =~ /^\d+$/ &&
+       $job->{'days'} =~ /^\d+$/ && $job->{'months'} eq '*' &&
+       $job->{'weekdays'} eq '*') {
+	return &text($pfx.'when_month',
+		     sprintf("%2.2d", $job->{'mins'}),
+		     sprintf("%2.2d", $job->{'hours'}),
+		     $job->{'days'});
 	}
-elsif ($_[0]->{'mins'} =~ /^\d+$/ && $_[0]->{'hours'} =~ /^\d+$/ && $_[0]->{'days'} eq '*' && $_[0]->{'months'} eq '*' && $_[0]->{'weekdays'} =~ /^\d+$/) {
-	return &text($pfx.'when_weekday', sprintf("%2.2d", $_[0]->{'mins'}), $_[0]->{'hours'}, $text{"day_".$_[0]->{'weekdays'}});
+elsif ($job->{'mins'} =~ /^\d+$/ && $job->{'hours'} =~ /^\d+$/ &&
+       $job->{'days'} eq '*' && $job->{'months'} eq '*' &&
+       $job->{'weekdays'} =~ /^\d+$/) {
+	return &text($pfx.'when_weekday',
+		     sprintf("%2.2d", $job->{'mins'}),
+		     sprintf("%2.2d", $job->{'hours'}),
+		     $text{"day_".$job->{'weekdays'}});
 	}
 else {
-	return &text($pfx.'when_cron', join(" ", $_[0]->{'mins'}, $_[0]->{'hours'}, $_[0]->{'days'}, $_[0]->{'months'}, $_[0]->{'weekdays'}));
+	return &text($pfx.'when_cron', join(" ", $job->{'mins'}, $job->{'hours'}, $job->{'days'}, $job->{'months'}, $job->{'weekdays'}));
 	}
 }
 
@@ -1594,7 +1556,8 @@ if ($err) {
 
 =head2 cleanup_temp_files
 
-Called from cron to delete old files in the Webmin /tmp directory
+Called from cron to delete old files in the Webmin /tmp directory, and also
+old lock links directories.
 
 =cut
 sub cleanup_temp_files
@@ -1604,27 +1567,64 @@ if (!$gconfig{'tempdelete_days'}) {
 	print STDERR "Temp file clearing is disabled\n";
 	return;
 	}
+
+# Cleanup files in /tmp/.webmin
 if ($gconfig{'tempdir'} && !$gconfig{'tempdirdelete'}) {
 	print STDERR "Temp file clearing is not done for the custom directory $gconfig{'tempdir'}\n";
-	return;
 	}
+else {
+	my $tempdir = &transname();
+	$tempdir =~ s/\/([^\/]+)$//;
+	if (!$tempdir || $tempdir eq "/") {
+		$tempdir = "/tmp/.webmin";
+		}
 
-local $tempdir = &transname();
-$tempdir =~ s/\/([^\/]+)$//;
-if (!$tempdir || $tempdir eq "/") {
-	$tempdir = "/tmp/.webmin";
-	}
-
-local $cutoff = time() - $gconfig{'tempdelete_days'}*24*60*60;
-opendir(DIR, $tempdir);
-foreach my $f (readdir(DIR)) {
-	next if ($f eq "." || $f eq "..");
-	local @st = lstat("$tempdir/$f");
-	if ($st[9] < $cutoff) {
-		&unlink_file("$tempdir/$f");
+	my $cutoff = time() - $gconfig{'tempdelete_days'}*24*60*60;
+	if (opendir(DIR, $tempdir)) {
+		foreach my $f (readdir(DIR)) {
+			next if ($f eq "." || $f eq "..");
+			my @st = lstat("$tempdir/$f");
+			if ($st[9] < $cutoff) {
+				&unlink_file("$tempdir/$f");
+				}
+			}
+		closedir(DIR);
 		}
 	}
-closedir(DIR);
+
+# Delete stale lock files
+my $lockdir = $var_directory."/locks";
+if (opendir(DIR, $lockdir)) {
+	foreach my $f (readdir(DIR)) {
+		next if ($f eq "." || $f eq "..");
+		next if ($f !~ /^\d+$/);
+		if (!kill(0, $f)) {
+			# Process is gone, but never cleaned up!
+			&unlink_file("$lockdir/$f");
+			}
+		}
+	closedir(DIR);
+	}
+
+# Cleanup old websockets
+foreach (&get_miniserv_websockets_modules()) {
+	&cleanup_miniserv_websockets(undef, $_);
+	}
+
+# Delete forgot-password files older than 1 day
+if (opendir(DIR, $main::forgot_password_link_dir)) {
+	my $cutoff = time() - 24*60*60;
+	foreach my $f (readdir(DIR)) {
+		next if ($f eq "." || $f eq "..");
+		next if ($f eq "ratelimit");
+		my $path = $main::forgot_password_link_dir."/".$f;
+		my @st = stat($path);
+		if ($st[9] < $cutoff) {
+			&unlink_file($path);
+			}
+		}
+	closedir(DIR);
+	}
 }
 
 =head2 list_cron_files()
@@ -1641,6 +1641,7 @@ if ($config{'system_crontab'}) {
 	}
 if ($config{'cronfiles_dir'}) {
 	push(@files, glob(&translate_filename($config{'cronfiles_dir'})."/*"));
+	@files = grep { -f $_ } @files;
 	}
 return &unique(@files);
 }
