@@ -1471,7 +1471,8 @@ return $data;
 }
 
 # simplify_date(datestring, [format])
-# Given a date from an email header, convert to the user's preferred format
+# Given a date from an email header, convert to the user's preferred format and
+# return it as an HTML string
 sub simplify_date
 {
 local ($date, $fmt) = @_;
@@ -1494,12 +1495,12 @@ if ($u) {
 		}
 	}
 elsif ($date =~ /^(\S+),\s+0*(\d+)\s+(\S+)\s+(\d+)\s+(\d+):(\d+)/) {
-	return "$2/$3/$4 $5:$6";
+	return &html_escape("$2/$3/$4 $5:$6");
 	}
 elsif ($date =~ /^0*(\d+)\s+(\S+)\s+(\d+)\s+(\d+):(\d+)/) {
-	return "$1/$2/$3 $4:$5";
+	return &html_escape("$1/$2/$3 $4:$5");
 	}
-return $date;
+return &html_escape($date);
 }
 
 # simplify_from(from)
@@ -2905,11 +2906,12 @@ if ($@) {
 return $rv;
 }
 
-# send_text_mail(from, to, cc, subject, body, [smtp-server])
+# send_text_mail(from, to, cc, subject, body, [smtp-server],
+# 		 [smtp-user, smtp-pass])
 # A convenience function for sending a email with just a text body
 sub send_text_mail
 {
-local ($from, $to, $cc, $subject, $body, $smtp) = @_;
+local ($from, $to, $cc, $subject, $body, $smtp, $user, $pass) = @_;
 local $cs = &get_charset();
 local $attach = 
 	{ 'headers' => [ [ 'Content-Type', 'text/plain; charset='.$cs ],
@@ -2921,7 +2923,7 @@ local $mail = { 'headers' =>
 		  [ 'Cc', $cc ],
 		  [ 'Subject', &encode_mimewords($subject) ] ],
 		'attach' => [ $attach ] };
-return &send_mail($mail, undef, 1, 0, $smtp);
+return &send_mail($mail, undef, 1, 0, $smtp, $user, $pass);
 }
 
 # make_from_line(address, [time])
