@@ -662,7 +662,7 @@ if [ ! -f "$config_dir/.pre-install" ]; then
 fi
 
 # Test if we have systemd system
-systemctlcmd=$(which systemctl 2>/dev/null)
+systemctlcmd=$(command -v systemctl 2>/dev/null || :)
 if [ -x "$systemctlcmd" ]; then
     initsys=$(cat /proc/1/comm 2>/dev/null)
     if [ "$initsys" != "systemd" ]; then
@@ -819,15 +819,29 @@ if [ "$upgrading" != 1 ]; then
 		echo licence_module=$licence_module >>$config_dir/config
 	fi
 
+	# Enable log rotation by default
+	echo "logclear=1" >> $config_dir/miniserv.conf
+	echo "logclear=1" >> $config_dir/config
+	
+	# Enable HSTS by default
+	echo "ssl_hsts=1" >> $config_dir/miniserv.conf
+
+	# Enable force redirect to SSL by default
+	echo "ssl_enforce=1" >> $config_dir/miniserv.conf
+
 	# Disallow unknown referers by default
 	echo "referers_none=1" >>$config_dir/config
 else
-	# one-off hack to set log variable in config from miniserv.conf
-	grep log= $config_dir/config >/dev/null
-	if [ "$?" = "1" ]; then
-		grep log= $config_dir/miniserv.conf >> $config_dir/config
-		grep logtime= $config_dir/miniserv.conf >> $config_dir/config
-		grep logclear= $config_dir/miniserv.conf >> $config_dir/config
+	# Enable HSTS by default if not set
+	grep ssl_hsts= $config_dir/miniserv.conf >/dev/null
+	if [ "$?" != "0" ]; then
+		echo "ssl_hsts=1" >> $config_dir/miniserv.conf
+	fi
+
+	# Enable force redirect to SSL if not set
+	grep ssl_enforce= $config_dir/miniserv.conf >/dev/null
+	if [ "$?" != "0" ]; then
+		echo "ssl_enforce=1" >> $config_dir/miniserv.conf
 	fi
 
 	# Disallow unknown referers if not set
